#include <BridgeAttr.h>
#include <RPCHandleSet.h>

namespace RPC2TangoBridge_ns
{

BridgeAttr::BridgeAttr(const char *name, long data_type,Tango::AttrWriteType w_type) :
                        Attr(name, data_type, w_type) 
{
	wrpch = NULL;
	rpch = RPCHandleSet::instance()->handleFromList(name);
	if(!rpch)
	{
		rpch = new RPCHandle(name);
		if(rpch && rpch->connected())
		{
			RPCHandleSet::instance()->addHandle(rpch);
			//printf("\e[1;32m*\e[0m connected to host \"%s\" port %d for attribute \"%s\"", rpch->host().c_str(), rpch->port(), name);
		}
		else if(rpch){
			//printf("\e[1;31m* \e[0merror creating RPC handle for \"%s\": \e[1;4;31mmessage\e[0m: \"%s\"\n", name, rpch->lastRpcError().c_str());
		}
	}
	else{
		//printf("\e[1;32m* \e[0mread handle for  attribute \"%s\" was already created (host \"%s\":%d\n", name, rpch->host().c_str(), rpch->port());
	}
	if(rpch && rpch->connected())
		d_connected = true;
	else
		d_connected = false;
	if(rpch)
	{
		rpch->_refcnt++;
		rrpch = new RPCReadHandle(rpch);
	}
}

/* read write attribute */
BridgeAttr::BridgeAttr(const char *name, const char *wname, long data_type,Tango::AttrWriteType w_type) :
                        Attr(name, data_type, w_type) 
{
	rpch = RPCHandleSet::instance()->handleFromList(name);
	if(!rpch)
	{
		rpch = new RPCHandle(name);
		if(rpch && rpch->connected())
			RPCHandleSet::instance()->addHandle(rpch);
		else if(rpch){
			//printf("\e[1;31m* \e[0merror creating RPC handle for \"%s\": \e[1;4;31mmessage\e[0m: \"%s\"\n", 
					//name, rpch->lastRpcError().c_str());
		}
	}
	else{
		//printf("\e[1;32m* \e[0mread handle for  attribute \"%s\" was already created (host \"%s\":%d\n", name, rpch->host().c_str(), rpch->port());
	}
	
	if(rpch && rpch->connected())
		d_connected = true;
	else
		d_connected = false;
	if(rpch)
	{
		rpch->_refcnt++;
		rrpch = new RPCReadHandle(rpch);
		wrpch = new RPCWriteHandle(rpch);
	}
	d_writeName = string(wname);
}


SpectrumBridgeAttr::SpectrumBridgeAttr(const char *name,long data_type,Tango::AttrWriteType w_type, long max_x)
	: SpectrumAttr(name, data_type, w_type, max_x)
{
	rpch = RPCHandleSet::instance()->handleFromList(name);
	if(!rpch)
	{
		rpch = new RPCHandle(name);
		if(rpch && rpch->connected())
			RPCHandleSet::instance()->addHandle(rpch);
		else if(rpch){
			//printf("\e[1;31m* \e[0merror creating RPC handle for SPECTRUM \"%s\": \e[1;4;31mmessage\e[0m: \"%s\"\n", name, rpch->lastRpcError().c_str());
		}
	}
	else{
		//printf("\e[1;32m* \e[0mread handle for spectrum attribute \"%s\" was already created (host \"%s\":%d\n", name, rpch->host().c_str(), rpch->port());
	}
	if(rpch && rpch->connected())
		d_connected = true;
	else
		d_connected = false;
	if(rpch)
	{
		rpch->_refcnt++;
		rrpch =new  RPCReadHandle(rpch);
	}
}

std::string BridgeAttr::host()
{
	if(rpch && rpch->connected())
		return rpch->host();
	return string("unconnected");
}

unsigned short BridgeAttr::port()
{
	if(rpch && rpch->connected())
		return rpch->port();
	return 0;
}

string SpectrumBridgeAttr::host()
{
	if(rpch && rpch->connected())
		return rpch->host();
	return string("unconnected");
}

unsigned short SpectrumBridgeAttr::port()
{
	if(rpch && rpch->connected())
		return rpch->port();
	return 0;
}


/* destructors: called by delete_device() */
SpectrumBridgeAttr::~SpectrumBridgeAttr()
{
	if(rpch)
		rpch->_refcnt--;
	if(rpch && !rpch->_refcnt)
	{
		//printf("\e[1;32m x\e[0m deleting rpc handle linked to spectrum attribute \"%s\"\n", get_name().c_str());
		RPCHandleSet::instance()->removeHandle(rpch);
		delete rpch;
	}
	else{
		//printf("\e[1;35m *\e[0m rpc handle linked to \"%s\"  still referenced by %d attributes, not freeing\n", get_name().c_str(), rpch->_refcnt);
	}
}

BridgeAttr::~BridgeAttr()
{
	if(rpch)
		rpch->_refcnt--;
	if(rpch && !rpch->_refcnt)
	{
		//printf("\e[1;32m x\e[0m deleting rpc handle linked to scalar attribute \"%s\"\n", get_name().c_str());
		RPCHandleSet::instance()->removeHandle(rpch);
		delete rpch;
	}
	else{
		//printf("\e[1;35m *\e[0m rpc handle linked to \"%s\"  still referenced by %d attributes, not freeing\n", get_name().c_str(), rpch->_refcnt);
	}
}

} /* namespace ends */




