//+=============================================================================
//
// file :         RPC2TangoBridge.cpp
//
// description :  C++ source for the RPC2TangoBridge and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                RPC2TangoBridge are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: claudio $
//
// $Revision: 1.6 $
//
// $Log: RPC2TangoBridge.cpp,v $
// Revision 1.6  2017-03-28 09:39:02  claudio
// ticket 2413: tango-9
//
// Revision 1.5  2012-08-09 13:04:39  claudio
// tango 4 impl
//
// Revision 1.4  2012-03-12 11:31:06  claudio
// tango-7.2.6
// rimossi "printf" ridondanti
//
// Revision 1.3  2009/08/19 07:37:57  giacomo
// Correzioni al bridge e allineamento set point a read point in ogni
// read attribute (per gli attributi RW).
//
// Revision 1.2  2009/02/26 08:15:45  giacomo
// Versione di RPC2TangoBridge con supporto ad attributi SCALARI READ WRITE.
// La parte WRITE va specificata nelle device properties separata da : dalla
// parte rpc read.
//
// Revision 1.1.1.1  2008/04/28 09:13:16  giacomo
// Prima versione di RPC2TangoBridge
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name       |  Method's name
//	----------------------------------------
//  State                |  dev_state()
//  Status               |  dev_status()
//  RPCDataForAttribute  |  rpcdata_for_attribute()
//  Standby              |  standby()
//  On                   |  on()
//
//===================================================================


#include <tango.h>
#include <RPC2TangoBridge.h>
#include <RPC2TangoBridgeClass.h>
#include <RPC2TangoBridgeThread.h>
#include <BridgeAttr.h>
#include <RpcHandle.h>
#include <RPCHandleSet.h>
#include <attribute_converter.h>
#include <errno.h>


using namespace Tango;
using namespace std;

namespace RPC2TangoBridge_ns
{

//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::RPC2TangoBridge(string &s)
// 
// description : 	constructor for simulated RPC2TangoBridge
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
RPC2TangoBridge::RPC2TangoBridge(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

RPC2TangoBridge::RPC2TangoBridge(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

RPC2TangoBridge::RPC2TangoBridge(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------

void RPC2TangoBridge::delete_device()
{
	//	Delete device's allocated object
	
	set_state(Tango::CLOSE);
	set_status("deallocating attributes and freeing resources");

	map<string, BridgeAttr* >::iterator mapiter;
	/* NOTE: tango: device.c if  an attribute already exists, the new attribute is deleted!
	 * So: remove all attributes in delete device.
	 */
	for(mapiter = namesMap.begin(); mapiter != namesMap.end(); mapiter++)
	{
		//printf("\e[1;32mx\e[0m removing scalar dynamic attribute \"%s\"\n", (*mapiter).second->get_name().c_str());
		try{
			remove_attribute((*mapiter).second, true);
		}
		catch(Tango::DevFailed &e)
		{
			//printf("\e[1;31m*\e[0m error removing attribute: \"%s\"", (*mapiter).second->get_name().c_str());
			Tango::Except::print_exception(e);
		}
	}
 	namesMap.erase(namesMap.begin(), namesMap.end());
	
	map<string, SpectrumBridgeAttr* >::iterator spmapiter;
	for(spmapiter = spectrumNamesMap.begin(); spmapiter != spectrumNamesMap.end(); spmapiter++)
	{
		//printf("\e[1;32mx\e[0m removing spectrum dynamic attribute \"%s\"\n", (*spmapiter).second->get_name().c_str());
		try{
			remove_attribute((*spmapiter).second, true);	
		}
		catch(Tango::DevFailed &e)
		{
			//printf("\e[1;31m* \e[0merror removing attribute: \"%s\"", (*mapiter).second->get_name().c_str());
			Tango::Except::print_exception(e);
		}
	}
	spectrumNamesMap.erase(spectrumNamesMap.begin(), spectrumNamesMap.end());
	
	/* do not clear() RPCHandleSet here: if we restart device the order of  attribute re-creation and destruction would 
	 * make the server crash. Indeed, the reference count of the handle would be 0 and so freed in an unappropriate 
	 * instant. On the other hand, if the server is restarted, we do not exit and the set remains alive and initialized.
	*/
}

//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void RPC2TangoBridge::init_device()
{
	set_state(Tango::INIT);
	INFO_STREAM << "RPC2TangoBridge::RPC2TangoBridge() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	set_state(Tango::INIT);
	set_status("Reading device properties");
	get_device_property();

	/* Now let's allocate the dynamic attributes whose name 
	 * is taken from the configuration file.
	 */
	set_status("Allocating dynamic attributes");
    inserted = 0;
    failed = 0;

	AllocateAttributes();
    if (inserted == 0){
        set_state(Tango::FAULT);
        set_status("failed to create all dynamic attributes");
    }
    else{
	set_state(Tango::ON);
    char mesg[90];
    sprintf(mesg,"created attributes : %d , failed: %d \n",inserted,failed);
    string sts(mesg);
	set_status(sts);
    }
}


//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void RPC2TangoBridge::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Rpc_items"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	RPC2TangoBridgeClass	*ds_class =
		(static_cast<RPC2TangoBridgeClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize Rpc_items from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rpc_items;
	//	Try to initialize Rpc_items from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  rpc_items;
	//	And try to extract Rpc_items value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rpc_items;



	//	End of Automatic code generation
	//------------------------------------------------------------------
	DEBUG_STREAM << rpc_items.size() << "rpc items " <<  endl;

}

/* NOTE: careful: add_attribute deletes an attribute with the same name if it was 
 * already created!
 */
void RPC2TangoBridge::AllocateAttributes()
{
	//int i = 0, totatt = 0, j;
	char rpcentity[MAXLEN];

	UserDefaultAttrProp default_att_prop;
	
// 	printf("\e[1;32m* \e[0mCreating dynamic attributes...\n");
// 	printf("\e[1;33m*\e[0m If you want to remove an attribute from the\n"
// 		"\e[1;33m*\e[0m list in the \e[1;4;37mdevice properties\e[0m, remember\n"
// 		"\e[1;33m*\e[0m first to \e[1;4;37mdisable polling\e[0m and to restore\n"
// 		"\e[1;33m*\e[0m any modified attribute property to its default.\n"
// 		"\e[1;33m*\e[0m Otherwise you will not be able to start this server\n"
// 		"\e[1;33m*\e[0m until references to no more existing attributes remain\n"
// 		"\e[1;33m*\e[0m on the \e[1;4;37mtango database\e[0m.\n\n");

	/* rpc_items is a vector of strings, got from init_device and loaded
	 * from the DeviceProperties.
	 */
// 	totatt = rpc_items.size();
// 
// 	if(totatt == 0)
// 	{
// // 		printf("\e[1;31mNo RPC entities specified as device properties\n\e[0m");
// // 		printf("\e[1;32mhint\e[0m: Define some entries and restart the device server\n\n");
// 		//exit(EXIT_SUCCESS);
// 	}

	for(unsigned int i = 0; i < rpc_items.size(); i++)
	{
// 		printf("\n- rpc_items %d:\"%s\"\n", i, rpc_items[i].c_str());
		memset(rpcentity, 0, MAXLEN);
		strncpy(rpcentity, rpc_items[i].c_str(), MAXLEN - 1);
// 		print_progress(totatt, i);

		/*bool force_is_spectrum = */force_spectrum(rpcentity);
		map<string, BridgeAttr*> ::iterator scalarMapIter = namesMap.find(string(rpcentity));
		map<string, SpectrumBridgeAttr*> ::iterator spectrumMapIter = spectrumNamesMap.find(string(rpcentity));
		if( scalarMapIter != namesMap.end() || spectrumMapIter != spectrumNamesMap.end() )
		{
			/* WARNING: do not add_attribute() two times with the same NAME! */
			//printf("\e[1;31m* \e[0mthe element \"\e[1;31m%s\e[0m\" is duplicated: not adding it again\n", rpcentity);
		}
		else
		{
			try
			{
				AttributeConverter tangoAttr(rpcentity);
				if(tangoAttr.isSpectrum())
				{
					SpectrumBridgeAttr* spattr = tangoAttr.spectrumAttribute();
					string name(spattr->get_name());
					if(spattr != NULL && spattr->connected())
					{
						DEBUG_STREAM << "Aggiunto alla mappa con nome " << name << " e " << spattr->get_name() << " 0x" << spattr << endl;
						spectrumNamesMap[name] = spattr;
						add_attribute(spattr);
                        inserted++;
					}
					else failed++;
					
				}
				else
				{
					BridgeAttr * att = tangoAttr.scalarAttribute();
					string name(att->get_name());
					if(att != NULL && att->connected())
					{
						/* insert the couple attribute name/BridgeAttr* into the names map */
						namesMap[name] = att; 
						add_attribute(att);
                        inserted++;
					}
					else { ERROR_STREAM  << "failed: " << name << endl;failed++;}
					
				}
			}
			catch(Tango::DevFailed &e)
			{
                failed++;
				printException("Error creating attribute", e);
			}
		} /* else: element not already present in the map */
	}
    RPCHandleSet::instance()->handleCount();
}

//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void RPC2TangoBridge::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void RPC2TangoBridge::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "RPC2TangoBridge::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;

//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		RPC2TangoBridge::read_AlarmAttribute
// 
// description : 	Extract real attribute values for AlarmAttribute acquisition result.
//
//-----------------------------------------------------------------------------
void RPC2TangoBridge::read_BridgeAttr(Tango::Attribute &attr)
{
	DEBUG_STREAM << "RPC2TangoBridge::read_AlarmAttribute(Tango::Attribute &attr) entering... "<< endl;

	read_dynamic_attribute(attr);
}

void RPC2TangoBridge::write_BridgeAttr(Tango::WAttribute &attr)
{
	bool res;
	DEBUG_STREAM<<"write_BridgeAttr()" << endl;
	res = write_dynamic_attribute(attr);
	if(!res)
		Except::throw_exception("Error writing attribute " + attr.get_name(),
				"Probably the format is not supported.\nSupported formats: long, double, short, boolean - SCALARS -",
				"RPC2TangoBridge::write_BridgeAttr()");
}

void RPC2TangoBridge::print_progress(int tot, int part)
{
        return;
//         const int numbars_tot = 20;
//         int current_bar = part * 20 / tot;
//         int i = 0;
//         int percent = part * 100 / tot;
//         printf("\r             \r");
//         //printf("\r\b\b\b\b\b\b\b\b\b\b\b\b\r");
//         fflush(stdout);
//         if(part < tot)
//         {
//                 if(part % 4 == 0)
//                         printf("|");
//                 else if(part % 4 == 1)
//                         printf( "\\");
//                 else if(part % 4 == 2)
//                         printf("-");
//                 else if(part % 4 == 3)
//                         printf("/");
//                 fflush(stdout);
//                 printf("\t[%d%%]", percent);
//         }
//         fflush(stdout);
}

void RPC2TangoBridge::printException(const char* msg, Tango::DevFailed &e)
{
	string err, descr, orig;
        int i;
        for(i = e.errors.length() - 1; i >= 0;  i--)
        {
                err = e.errors[i].reason;
                descr = e.errors[i].desc;
                orig = e.errors[i].origin;
	}
// 	printf("\n* \e[1;31m%s:\e[0m\n\t%s\n\t%s\n\t%s\n\e[0m", msg, err.c_str(), 
// 		descr.c_str(), orig.c_str());
}

//+------------------------------------------------------------------
/**
 *	method:	RPC2TangoBridge::rpcdata_for_attribute
 *
 *	description:	method to execute "RPCDataForAttribute"
 *	[3]the connection state (connected,disconnected)
 *
 * @param	argin	The name of an attribute representing an RPC call, such as SIP120_S2.1_PRES_RR
 * @return	Three values: [1]the RPC host name; [2]the RPC port; [3]the connection state (connected,disconnected)
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *RPC2TangoBridge::rpcdata_for_attribute(Tango::DevString argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();
	DEBUG_STREAM << "RPC2TangoBridge::rpcdata_for_attribute(): entering... !" << endl;

	//	Add your own code to control device here
	bool found = true;
	map<string, BridgeAttr*>::iterator smapiter;
	RPCHandle *handle = NULL;
	map<string, SpectrumBridgeAttr*>::iterator spmapiter;
	char port[8];

	for(smapiter= namesMap.begin(); smapiter != namesMap.end(); smapiter++)
	{
		if(!strcmp(smapiter->first.c_str(), argin))
		{
			handle = smapiter->second->rpcHandle();
			break;
		}
	}
	for(spmapiter= spectrumNamesMap.begin(); spmapiter != spectrumNamesMap.end(); spmapiter++)
	{
		if(!strcmp(spmapiter->first.c_str(), argin))
		{
			handle = spmapiter->second->rpcHandle();
			break;
		}
	}
	if(smapiter == namesMap.end() && spmapiter == spectrumNamesMap.end())
		found = false;

	/* could avoid found variable and test for NULL, but this gives the possibility
	 * to have an attribute with a handle NULL
	 */
	if(!found)
	{
		argout->length(1);
		(*argout)[0] = CORBA::string_dup("No attribute with that name!");
	}
	else if(handle == NULL)
	{
		argout->length(1);
		(*argout)[0] = CORBA::string_dup("invalid handle: this won't work");
	}
	else
	{
		argout->length(3);
		(*argout)[0] = CORBA::string_dup(handle->host().c_str());
		snprintf(port, 8, "%d", handle->port());
		(*argout)[1] = CORBA::string_dup(port);
		if(handle->connected())
			(*argout)[2] = CORBA::string_dup("connected");
		else
			(*argout)[2] = CORBA::string_dup("disconnected");
	}		
	
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	RPC2TangoBridge::standby
 *
 *	description:	method to execute "Standby"
 *	Puts the server in Standby mode: RPC calls are disabled.
 *
 *
 */
//+------------------------------------------------------------------
void RPC2TangoBridge::standby()
{
	DEBUG_STREAM << "RPC2TangoBridge::standby(): entering... !" << endl;

	//	Add your own code to control device here
	set_state(Tango::STANDBY);
	set_status("Standby mode: no RPC calls will be made");

}

bool RPC2TangoBridge::is_BridgeAttr_allowed(Tango::AttReqType type)
{
	if(get_state() != Tango::ON)
		return false;
	else 
		return true;
}

//+------------------------------------------------------------------
/**
 *	method:	RPC2TangoBridge::on
 *
 *	description:	method to execute "On"
 *	From the Standby state, the command On enables the RPC calls again.
 *
 *
 */
//+------------------------------------------------------------------
void RPC2TangoBridge::on()
{
	DEBUG_STREAM << "RPC2TangoBridge::on(): entering... !" << endl;

	//	Add your own code to control device here
	set_state(Tango::ON);
	set_status("On: RPC calls enabled");
}



bool RPC2TangoBridge::force_spectrum(char *s)
{
	unsigned  int i;
	for(i = 0; i < strlen(s); i++)
		if(s[i] == ',')
			break;
	if(i == strlen(s))
		return false;
	if(strncmp(s + i + 1, "spectrum", strlen("spectrum")) == 0)
	{
		s[i] = '\0'; /* remove the spectrum force keyword */
		return true;
	}
	return false;

}

}	//	namespace
