//=============================================================================
//
// file :        RPC2TangoBridge.h
//
// description : Include for the RPC2TangoBridge class.
//
// project :	Provides a conversion from RPC alarms to Tango alarm attributes.
//
// $Author: claudio $
//
// $Revision: 1.4 $
//
// $Log: RPC2TangoBridge.h,v $
// Revision 1.4  2017-03-28 09:39:02  claudio
// ticket 2413: tango-9
//
// Revision 1.3  2012-08-09 13:04:39  claudio
// tango 4 impl
//
// Revision 1.2  2009/02/26 08:15:45  giacomo
// Versione di RPC2TangoBridge con supporto ad attributi SCALARI READ WRITE.
// La parte WRITE va specificata nelle device properties separata da : dalla
// parte rpc read.
//
// Revision 1.1.1.1  2008/04/28 09:13:16  giacomo
// Prima versione di RPC2TangoBridge
//
//
// copyleft :    European Synchrotron Radiation Facility
//               BP 220, Grenoble 38043
//               FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
#ifndef _RPCALARMTANGO_H
#define _RPCALARMTANGO_H

#include <tango.h>
#include <map>
//#include "RPC2TangoBridgeThread.h"


using namespace Tango;
using namespace std;

/**
 * @author	$Author: claudio $
 * @version	$Revision: 1.4 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------


namespace RPC2TangoBridge_ns
{

class RPC2TangoBridgeThread;
class BridgeAttr;
class SpectrumBridgeAttr;
class RPCHandle;

/**
 * Class Description:
 * Every RPC client who emits alarms will send them to this server.
 *	RPCAlarmTAngo will then map every alarm into a specific Tango attribute, so
 *	that it can be notified to the Tango Alarm System.
 *	The Tango Alarm System has to subscribe to all the attributes managed by this
 *	RPC2TangoBridge.
 */

/*
 *	Device States Description:
*  Tango::ON :       The device is in on state: the dynamic attributes have been allocated and
 *                    the server is ready for read operations on the RPC handles.
*  Tango::INIT :     The server is allocating the dynamic attributes.
 *                    The server is reading the device property named Rpc_items and for each line
 *                    is allocating a new dynamic attribute that maps the RPC call into a READ ONLY
 *                    tango attribute.
*  Tango::FAULT :    A read attribute failed.
 *                    This can be a transitional state, when an RPC call fails for instance, and so
 *                    it can appear for a short time and then resolve to the ON state when the next
 *                    read ends successfully.
 *                    Anyway, a persistent state of FAULT can indicate network problems or
 *                    continuous failures in reads.
*  Tango::CLOSE :    Deallocating dynamic attributes and exiting
*  Tango::STANDBY :  In this state the RPC calls are disabled.
 */


class RPC2TangoBridge: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//----------------------------------------
//	void read_AlarmAttribute(Attribute attr);
//	void write_AlarmAttribute(WAttribute &att);
//	bool is_AlarmAttribute_allowed(Tango::AttReqType ty);
	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	Here you have to put the list of rpc items.
 *	An RPC item is made up of RPCNAME:RPCHOST:RPCPORT, for instance:
 *	PSB_T2.2_STAT_RF:rpcpsb.domain.com:9100
 */
	vector<string>	rpc_items;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	RPC2TangoBridge(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	RPC2TangoBridge(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	RPC2TangoBridge(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~RPC2TangoBridge() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name RPC2TangoBridge methods prototypes
 */

//@{
/**
 *	Execution allowed for RPCDataForAttribute command.
 */
	virtual bool is_RPCDataForAttribute_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Standby command.
 */
	virtual bool is_Standby_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for On command.
 */
	virtual bool is_On_allowed(const CORBA::Any &any);
/**
 * [3]the connection state (connected,disconnected)
 *	@param	argin	The name of an attribute representing an RPC call, such as SIP120_S2.1_PRES_RR
 *	@return	Three values: [1]the RPC host name; [2]the RPC port; [3]the connection state (connected,disconnected)
 *	@exception DevFailed
 */
	Tango::DevVarStringArray	*rpcdata_for_attribute(Tango::DevString);
/**
 * Puts the server in Standby mode: RPC calls are disabled.
 *	@exception DevFailed
 */
	void	standby();
/**
 * From the Standby state, the command On enables the RPC calls again.
 *	@exception DevFailed
 */
	void	on();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	
	/* Returns the configuration */
////	map<const char*, struct alarm_tango_info*>  *TangoAlarmMap() { return &tangoalmap; }

	 /**
	 *	Read/Write allowed for BridgeAttr attribute.
	 */
	virtual bool is_BridgeAttr_allowed(Tango::AttReqType type);

	/**
	 *	Extract real attribute values for BridgeAttr acquisition result.
	 */
	virtual void read_BridgeAttr(Tango::Attribute &attr);
	
	/**
 	*      Write attribute values to hardware.
 	*/
        virtual void write_BridgeAttr(Tango::WAttribute &attr);


	void read_attr_hardware(std::vector<long int, std::allocator<long int> >&);

protected :	
	//	Add your own data members here
	//-----------------------------------------
	
		
	Tango::DevShort btest1, btest2;		
	RPC2TangoBridgeThread *rpcthread;
private:
	void AllocateAttributes();

	void print_progress(int tot, int part);
	
	map<string, BridgeAttr*> namesMap;
	map<string, SpectrumBridgeAttr*> spectrumNamesMap;

	map<BridgeAttr*, const char*> lpcHostsMap;
	
	void read_dynamic_attribute(Tango::Attribute &attr);
	bool write_dynamic_attribute(Tango::WAttribute &w);
	void printException(const char* msg, Tango::DevFailed &e);
	bool force_spectrum(char* device_property);
    
    //coutrer of ok/failed
    unsigned int inserted;
    unsigned int failed;
	
//	Tango::DevShort *attr_AlarmAttribute_read;
};

}	// namespace_ns

#endif	// _RPCALARMTANGO_H
