#include <RpcHandle.h>
#include <omnithread.h>
#include <RpcResolver.h>
#include <errtype.h>
#include <cstdlib>

#define MAXERRLEN	512

using namespace std;

namespace RPC2TangoBridge_ns
{

static omni_mutex rpcMutex;

RPCHandle::RPCHandle(RPCHandle *other)
{
	_host = other->host();
	_port = other->port();
	_sid = other->sid();
	_refcnt = other->_refcnt;
	_lastRpcErr = lastRpcError();
	
}

RPCHandle::RPCHandle(string rpccall)
{
	char *host = 0;
	short int port = 0;
	_host = "";
	_port = 0;
	_sid = -1;
	_refcnt = 0;
	if(resolve_rpc(rpccall.c_str(), &host, &port))	
	{
		_host = string(host);
		free(host);
		_port = (unsigned short) port;
		if(!connect())
			_lastRpcErr = "Connection error to host " + _host;
		else
			_lastRpcErr = std::string("No error");
	}
	else /* do not connect(): this will leave _sid to -1 */
		_lastRpcErr = "failed resolving call " + rpccall;
    
	 
}

bool RPCHandle::resolve_rpc(const char *call, char** host, short int *port)
{
    int res;
    char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
    
    
    try{
        RpcResolver resolver;
        if(split_rpccall(call, fam, mem, act, mode) >= 0)
        {
            res = resolver.resolve(fam, mem, act, mode, host, port);
            if(res >= 0 && host != NULL) return true;
            else return false;
        }
    }
    catch (...){
        return false;
    }
}

RPCHandle::~RPCHandle()
{
	if(_sid >= 0)
	{
		//printf("\e[1;32m x\e[0m freeing RPC resource associated with host \"%s\" and port %d\n",_host.c_str(), _port);
		free_lpc(_sid);
		_sid = -1;
	}
	else{
		//printf("\e[1;31m !\e[0m not freeing RPC resource associated with host \"%s\" and port %d\n",_host.c_str(), _port);
	}
}

bool RPCHandle::lockRpc()
{
	rpcMutex.acquire();
	return 1;
}

bool RPCHandle::unlockRpc()
{
	rpcMutex.release();
	return 1;
}

/* this is private */
bool RPCHandle::connect()
{
	if((_sid = import_lpc(_host.c_str(), _port)) >= 0)
		return true;
	else
		return false;
}

bool RPCHandle::connected()
{
	if(_sid >= 0)
		return true;
	else
		return false;
}

int RPCHandle::split_rpccall(const char *call, char* fam, char *mem, char* act, char* mode)
{
    int i = 0, delim1, delim2, delim3;
	int underline_cnt = 0;
	int calllen = strlen(call);

	delim1 = delim2 = delim3 = -1;
	memset(fam, 0, RPCSPLITLEN);
	memset(mem, 0, RPCSPLITLEN);
	memset(act, 0, RPCSPLITLEN);
	memset(mode, 0, RPCSPLITLEN);

	for(i = 0; i < strlen(call) && i < MAXLEN; i++)
		if(call[i] == '_')
			underline_cnt++;

	if(underline_cnt != 3)
	{
		//printf("\e[1;31m*\e[0m Bad name  \"%s\"\n""  RPC calls must be in the form FAMILY_MEMBER_ACTION_MODE,\n""  separated by underline (\"_\")", call);
		return -1;
	}

	for(i = 0; i < strlen(call) && i < MAXLEN; i++)
	{
		if(call[i] == '_')
		{
			if(delim1 < 0)
				delim1 = i;
			else if(delim2 < 0)
				delim2 = i;
			else if(delim3 < 0)
				delim3 = i;
		}
	}
	if(delim2 - delim1 >= RPCSPLITLEN || delim3 - delim2 >= RPCSPLITLEN || calllen - delim3 >= RPCSPLITLEN ||
		delim1 >= RPCSPLITLEN)
	{
		//printf("\t\e[1;31m* \e[0mRPC call subparts too long in \"%s\"\n\t""  \e[1;4;35minfo\e[0m: maximum length: %d\n",call, RPCSPLITLEN);
		return -1;
	}

	/* should be alright here: */
	strncpy(fam, call, delim1);
	strncpy(mem, call + delim1 + 1, delim2 - delim1 - 1);
	strncpy(act, call + delim2 + 1, delim3 - delim2 - 1);
	strncpy(mode, call + delim3 + 1, calllen - delim3 - 1);

	return 0;
}



}

