#include "RpcReadHandle.h"

using namespace std;

namespace RPC2TangoBridge_ns
{

RPCReadHandle::RPCReadHandle(RPCHandle* h) : RPCHandle(h)
{
	
}

idl_long_int RPCReadHandle::read_long(const char *callname, int* ok)
{
	idl_long_int ret;
	char error[MAXERRLEN];
	long res = -1;
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RI") || !strcmp(mode, "RII"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RI"))
			res = lpc_RI(fam, mem, act, mode, &ret);
		else if(!strcmp(mode, "RII"))
				res = lpc_RII(fam, mem, act, mode, &ret);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return ret;

}

double RPCReadHandle::read_double(const char *callname, int* ok)
{
	double ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RR") || !strcmp(mode, "RRI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RR"))
			res = lpc_RR(fam, mem, act, mode, &ret);
		else if(!strcmp(mode, "RRI"))
				res = lpc_RRI(fam, mem, act, mode, &ret);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return ret;

}

unsigned char RPCReadHandle::read_uchar(const char *callname, int* ok)
{
	unsigned char ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RE") || !strcmp(mode, "REI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RE"))
			res = lpc_RE(fam, mem, act, mode, &ret);
		else if(!strcmp(mode, "REI"))
			res = lpc_REI(fam, mem, act, mode, &ret);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return ret;

}

idl_short_int RPCReadHandle::read_short(const char *callname, int* ok)
{
	idl_short_int ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RS") || !strcmp(mode, "RSI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RS"))
			res = lpc_RS(fam, mem, act, mode, &ret);
		else if(!strcmp(mode, "RSI"))
			res = lpc_RSI(fam, mem, act, mode, &ret);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return ret;

}

string RPCReadHandle::read_string(const char *callname, int* ok)
{
	string ret;
	long res = -1;
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	char error[MAXERRLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
        char* strdata;
        //idata.length = 0;
        //idata.sequence = NULL;
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RM") || !strcmp(mode, "RMI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RM"))
			res = lpc_RM(fam, mem, act, mode, &strdata);
		else if(!strcmp(mode, "RMI"))
			res = lpc_RMI(fam, mem, act, mode, &strdata);
		/* else: res remains initialized to -1 and will fail */
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
			 ret = "There was an error reading the value";
		}
	}
	/* release lock */
	// unlockRpc();
	ret = string(strdata);
        return ret;

}

unsigned char RPCReadHandle::read_boolean(const char *callname, int* ok)
{
	unsigned char ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RB") || !strcmp(mode, "RBI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RB"))
			res = lpc_RB(fam, mem, act, mode, &ret);
		else if(!strcmp(mode, "RBI"))
			res = lpc_RBI(fam, mem, act, mode, &ret);
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return ret;

}

intArray RPCReadHandle::read_slong(const char *callname, int* ok)
{
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	char error[MAXERRLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	long res = -1;
        intArray idata;
        idata.length = 0;
        idata.sequence = NULL;
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RAI") || !strcmp(mode, "RAII"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RAI"))
			res = lpc_RAI(fam, mem, act, mode, &idata);
		else if(!strcmp(mode, "RAII"))
			res = lpc_RAII(fam, mem, act, mode, &idata);
		/* else: res remains initialized to -1 and will fail */
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return idata;
}

realArray RPCReadHandle::read_sdouble(const char *callname, int* ok)
{
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	char error[MAXERRLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	long res = -1;
        realArray rdata;
        rdata.length = 0;
        rdata.sequence = NULL;
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RAR") || !strcmp(mode, "RAIR"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RAR"))
			res = lpc_RAR(fam, mem, act, mode, &rdata);
		else if(!strcmp(mode, "RAIR"))
			res = lpc_RAIR(fam, mem, act, mode, &rdata);
		/* else: res remains initialized to -1 and will fail */
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return rdata;

}

shortArray RPCReadHandle::read_sshort(const char *callname, int* ok)
{
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	char error[MAXERRLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	long res = -1;
        shortArray rdata;
        rdata.length = 0;
        rdata.sequence = NULL;
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RAS") || !strcmp(mode, "RAIS"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RAS"))
			res = lpc_RAS(fam, mem, act, mode, &rdata);
		else if(!strcmp(mode, "RAIS"))
			res = lpc_RAIS(fam, mem, act, mode, &rdata);
		/* else res remains initialized to -1 and will fail */
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return rdata;

}

charArray RPCReadHandle::read_schar(const char *callname, int* ok)
{
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	char error[MAXERRLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	long res = -1;
        charArray rdata;
        rdata.length = 0;
        rdata.sequence = NULL;
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RAC") || !strcmp(mode, "RAIC"))
			bind_lpc(_sid);
		if(!strcmp(mode, "RAC"))
			res = lpc_RAC(fam, mem, act, mode, &rdata);
		else if(!strcmp(mode, "RAIC"))
			res = lpc_RAIC(fam, mem, act, mode, &rdata);
		/* else: res remains initialized to -1 and this will fail */	
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return rdata;

}

booleanArray RPCReadHandle::read_sboolean(const char *callname, int* ok)
{
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	char error[MAXERRLEN];
	*ok = 1;
	/* acquire lock! */
	// lockRpc();
	long res = -1;
        booleanArray rdata;
        rdata.length = 0;
        rdata.sequence = NULL;
	idl_long_int rfLong;
	
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		*ok = 0;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "RF") || !strcmp(mode, "RFI"))
		{
			bind_lpc(_sid);
			//printf("_sid: %d\n", _sid);
			/* long int interpreted as array of 32 booleans */
			if(!strcmp(mode, "RF"))
				res = lpc_RF(fam, mem, act, mode, &rfLong);
			else if(!strcmp(mode, "RFI"))
				res = lpc_RFI(fam, mem, act, mode, &rfLong);
			if(res >= 0) /* convert a long of 32 bit into a booleanArray */
			{
				rdata.length = 32;
				rdata.sequence = (idl_boolean *)malloc(sizeof(idl_boolean) * 32);
				for(int i = 0; i < 32; i++)
					rdata.sequence[i] = (rfLong >> i) & 1;
			}
			else
				_lastRpcErr = string("failed lpc_RF or lpc_RFI");
		}
		else if(!strcmp(mode, "RAB"))
		{
			bind_lpc(_sid);
			res = lpc_RAB(fam, mem, act, mode, &rdata);
		}
		else if(!strcmp(mode, "RAIB"))
		{
			bind_lpc(_sid);
			res = lpc_RAIB(fam, mem, act, mode, &rdata);
		}
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error reading rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 *ok = 0;
		}
	}
	/* release lock */
	// unlockRpc();
        return rdata;
}



} /* namespace */


